#INCLUDE "dbc.h"

*!*	<pdm_sc_yes/>
*!*	<pdm_dd_yes/>
******************************************************************
**
** DBC low level function
** 
** Version: 0.0.1
** 
** Author: Martin Jindra
** E-mail: martin.jindra@egservis.cz
** Date:   2005-07-22
** 
** Acknowledgments:
** Doug Henning: SPECIAL REPORT - Visual FoxPro Data Dictionary,
**               Pinnacle Publishing Inc.,
**               Kent,1995, ISBN 1-880935-41-4 
**
******************************************************************


*******************************************************************
PROCEDURE DBC_GetIDObject(lcAlias,lcType,lcObjName) && Get Object ID
* lcAlias   - Alias of opended DBC
* lcType    - Object Type
* lcObjName - Object Name

*!*	<pdm_sc_yes/>
*!*	<pdm_dd_yes/>

SELE (lcAlias) && Skip to DBC 
IF INLIST(lcType,_DBC_Field,_DBC_Index,_DBC_Relation)
   LOCAL lcParent,lii,lcObj
   lii=AT(".",lcObjName) && Find dot
   lcParent=PADR(UPPER(LEFT(lcObjName,lii-1)),128)
   lcObj=PADR(UPPER(SUBST(lcObjName,lii+1)),128)
   * Find Parent
   LOCATE FOR INLIST(ObjectType,_DBC_Table,_DBC_View) AND;
              UPPER(ObjectName)==lcParent  AND ! DELE()
   IF FOUND() && If found
      lii=ObjectID
      * Find object
      LOCATE FOR ParentID=lii AND ObjectType==lcType AND ;
                 UPPER(ObjectName)==lcObj AND ! DELE()
   ENDIF
ELSE
   LOCATE FOR ObjectType==lcType AND UPPER(ObjectName)==PADR(UPPER(lcObjName),128);
          AND ! DELE()
ENDIF
RETURN IIF(FOUND(),ObjectID,_DBC_ObjectNotFound)
ENDPROC

************************************************************************************
PROCEDURE DBC_EnumObjects() && Get list of object by type
LPARAMETERS lcAlias,lcType,liIDParent,laObjects
* lcAlias    - Alias of opended DBC
* lcType     - Object Type
* liIDParent - Parent ID
* @laObjects - Output array for Properties and their values

EXTERNAL ARRAY laObjects

*!*	<pdm_sc_yes/>
*!*	<pdm_dd_yes/>

IF EMPTY(lcType)
   RETURN _DBC_ErrParamFailed
ENDIF
LOCAL llFound,liCount,lii,liID
liCount=0
SELE (lcAlias) && Skip to DBC 
IF INLIST(lcType,_DBC_Field,_DBC_Index,_DBC_Relation)
   SET FILTER TO ParentID=liIDParent AND ObjectType==lcType AND ! DELE()
ELSE
   SET FILTER TO ObjectType==lcType AND ! DELE()
ENDIF
GO TOP
COUNT TO liCount
IF liCount>0
   DIME laObjects(liCount,2)
   GO TOP
   lii=0
   SCAN ALL
        lii=lii+1
        laObjects(lii,1)=OBJECTID
        laObjects(lii,2)=ALLT(OBJECTNAME)
   ENDSCAN
ENDIF
SET FILTER TO
RETURN liCount
ENDPROC

******************************************************************************************
PROCEDURE DBC_EnumProperty() && Get list of object properties
LPARAMETERS lcAlias,liIDObject,laProp
* lcAlias    - Alia sof opended DBC
* liIDObject - Object ID
* @laProp    - Output array for Properties and their values

*!*	<pdm_sc_yes/>
*!*	<pdm_dd_yes/>

EXTERNAL ARRAY laProp

LOCAL liStart,liLen,liLenID,lii
liStart=1 && First char

SELE (lcAlias) && Skip to DBC 
LOCATE FOR ObjectID=liIDObject && Find object
IF !FOUND() && If not found
   RETURN _DBC_ObjectNotFound && Get out
ENDIF
lii=0 && Reset counter
DO WHILE liStart<=LEN(Property)
   liLen=DFO_C4ToI(SUBS(Property,liStart,4)) && Lenght
   * This is lenght of Property ID
   liLenID=IIF(DFO_C2ToI(SUBS(Property,liStart+4,2))>256,2,1)

   lii=lii+1 && Increment counter
   DIME laProp(lii,3)
   * Property ID
   laProp(lii,1)=IIF(liLenID=1,ASC(SUBS(Property,liStart+4+2,liLenID)),;
                 DFO_C2ToI(SUBS(Property,liStart+4+2,liLenID)))
   * Convert DBC value to Value
   laProp(lii,2)=DBC_ConvertPropertyToValue(laProp(lii,1),;
                 SUBS(Property,liStart+3+2+liLenID+1,liLen-(3+2+liLenID+1)))
   liStart=liStart+liLen
ENDDO
RETURN lii
ENDPROC

******************************************************************************************
PROCEDURE DBC_RIInfo() && Get RI Info for Relation
LPARAMETERS lcAlias,liIDObject,luValue
* lcAlias    - Alias of opended DBC
* liIDObject - Object ID
* @luValue   - (output) RI Info

*!*	<pdm_sc_yes/>
*!*	<pdm_dd_yes/>

SELE (lcAlias) && Skip to DBC 
LOCATE FOR ObjectID=liIDObject && Find object
IF !FOUND() && If not found
   RETURN _DBC_ObjectNotFound && Get out
ENDIF
IF OBJECTTYPE#_DBC_Relation
   RETURN _DBC_ObjectNotType
ENDIF
luValue=RIINFO
RETURN _DBC_ObjectFound
ENDPROC

******************************************************************************************
PROCEDURE DBC_GetSP(lcAlias,liType,luValue) && Get Stored Procedure
* lcAlias  - Alias of opended DBC
* liType   - Type (Source code/ P-Code)
* @luValue - (output) Stored procedures source code/P-Code

*!*	<pdm_sc_yes/>
*!*	<pdm_dd_yes/>

SELE (lcAlias) && Skip to DBC 
* Find object
LOCATE FOR OBJECTTYPE=_DBC_DataBase AND;
       UPPER(ObjectName)==UPPER(IIF(liType=_DBC_SP_TSource,_DBC_SP_Source,_DBC_SP_FXP))
IF !FOUND() && If not found
   RETURN _DBC_ObjectNotFound && Get out
ENDIF
luValue=CODE
RETURN _DBC_ObjectFound
ENDPROC

******************************************************************************************
PROCEDURE DBC_SetSP(lcAlias,liType,luValue) && Set Stored Procedure
* lcAlias  - Alias of opended DBC
* liType   - Type (Source code/ P-Code)
* @luValue - (output) Stored procedures source code/P-Code

*!*	<pdm_sc_yes/>
*!*	<pdm_dd_yes/>

SELE (lcAlias) && Skip to DBC 
* Find object
LOCATE FOR OBJECTTYPE=_DBC_DataBase AND ;
       UPPER(ObjectName)==UPPER(IIF(liType=_DBC_SP_TSource,_DBC_SP_Source,_DBC_SP_FXP))
IF !FOUND() && If not found
   RETURN _DBC_ObjectNotFound && Get out
ENDIF
REPLACE CODE WITH luValue
RETURN _DBC_ObjectFound
ENDPROC

******************************************************************************************
PROCEDURE DBC_GetProperty() && Get Property's value
LPARAMETERS lcAlias,liIDObject,liIDProp,luValue
* lcAlias    - Alias of opended DBC
* liIDObject - Object ID
* liIDProp   - Property ID
* @luValue   - (output) Value of property

*!*	<pdm_sc_yes/>
*!*	<pdm_dd_yes/>

LOCAL liStart,liLen,liLenID
liStart=1 && First char

SELE (lcAlias) && Skip to DBC 
LOCATE FOR ObjectID=liIDObject && Find object
IF !FOUND() && If not found
   RETURN _DBC_ObjectNotFound && Get out
ENDIF
DO WHILE liStart<=LEN(Property)
   liLen=DFO_C4ToI(SUBS(Property,liStart,4)) && Lenght
   * This is lenght of Property ID
   liLenID=IIF(DFO_C2ToI(SUBS(Property,liStart+4,2))>256,2,1)

   * Is't finding property
   IF IIF(liLenID=1,ASC(SUBS(Property,liStart+4+2,liLenID)),;
          DFO_C2ToI(SUBS(Property,liStart+4+2,liLenID)))=liIDProp
      luValue=DBC_ConvertPropertyToValue(liIDProp,;
              SUBS(Property,liStart+3+2+liLenID+1,liLen-(3+2+liLenID+1)) )
      RETURN _DBC_ObjectFound
   ENDIF
   liStart=liStart+liLen
ENDDO
luValue=DBC_GetDefaultValue(liIDProp)
RETURN _DBC_ObjectFound
ENDPROC

******************************************************************************************
PROCEDURE DBC_SetProperty(lcAlias,liIDObject,liIDProp,luValue) && Set Value of property
* lcAlias    - Alias of opended DBC
* liIDObject - Object ID
* liIDProp   - Property ID
* luValue    -  Value

*!*	<pdm_sc_yes/>
*!*	<pdm_dd_yes/>

LOCAL liStart,liLen,liLenID
liStart=1 && First char

SELE (lcAlias) && Skip to DBC 
LOCATE FOR ObjectID=liIDObject && Find object
IF !FOUND() && If not found
   RETURN _DBC_ObjectNotFound && Get out
ENDIF

* Convert value to DBC value
luValue=DBC_ConvertValueToProperty(liIDProp,luValue)
DO WHILE liStart<=LEN(Property)
   liLen=DFO_C4ToI(SUBS(Property,liStart,4)) && Lenght
   * This is lenght of Property ID
   liLenID=IIF(DFO_C2ToI(SUBS(Property,liStart+4,2))>256,2,1)

   * Is't finding property
   IF IIF(liLenID=1,ASC(SUBS(Property,liStart+4+2,liLenID)),;
      DFO_C2ToI(SUBS(Property,liStart+4+2,liLenID)))=liIDProp
      REPL Property WITH STUFF(Property,liStart,liLen,;
                    DFO_IToC4(4+2+liLenID+LEN(luValue))+;
                    DFO_IToC2(IIF(liIDProp>256,2,1))+;
                    IIF(liIDProp>256,DFO_IToC2(liIDProp),CHR(liIDProp))+luValue)
      RETURN _DBC_ObjectFound
   ENDIF
   liStart=liStart+liLen
ENDDO
REPL Property WITH Property+DFO_IToC4(4+2+IIF(liIDProp>256,2,1)+;
                   LEN(luValue))+DFO_IToC2(IIF(liIDProp>256,2,1))+;
                   IIF(liIDProp>256,DFO_IToC2(liIDProp),CHR(liIDProp))+luValue
RETURN _DBC_ObjectFound
ENDPROC


******************************************************************************************
PROCEDURE DBC_ClearProperty(lcAlias,liIDObject,liIDProp) && Clear property
* lcAlias    - Alias of opended DBC
* liIDObject - Object ID
* liIDProp   - Property ID

*!*	<pdm_sc_yes/>
*!*	<pdm_dd_yes/>

LOCAL liStart,liLen,liLenID
liStart=1 && First char

SELE (lcAlias) && Skip to DBC 
LOCATE FOR ObjectID=liIDObject && Find object
IF !FOUND() && If not found
   RETURN _DBC_ObjectNotFound && Get out
ENDIF

DO WHILE liStart<=LEN(Property)
   liLen=DFO_C4ToI(SUBS(Property,liStart,4)) && Lenght
   * This is lenght of Property ID
   liLenID=IIF(DFO_C2ToI(SUBS(Property,liStart+4,2))>256,2,1)

   * Is't finding property
   IF IIF(liLenID=1,ASC(SUBS(Property,liStart+4+2,liLenID)),;
      DFO_C2ToI(SUBS(Property,liStart+4+2,liLenID)))=liIDProp
      REPL Property WITH STUFF(Property,liStart,liLen,"")
      EXIT
   ENDIF
   liStart=liStart+liLen
ENDDO

RETURN _DBC_ObjectFound
ENDPROC

******************************************************************************************
PROCEDURE DBC_ConvertPropertyToValue(liIDProp,lcValue) && Convert DBC value to Value
* liIDProp - Property ID
* luValue  - Property value

*!*	<pdm_sc_yes/>
*!*	<pdm_dd_yes/>

RETURN IIF(INLIST(liIDProp,_DBCID_Comment,_DBCID_Path,;
           _DBCID_RuleExpression,_DBCID_RuleText,_DBCID_InsertTrigger,;
           _DBCID_UpdateTrigger,_DBCID_DeleteTrigger,_DBCID_PrimaryKey,;
           _DBCID_DefaultValue,_DBCID_Caption,_DBCID_ChildTag,;
           _DBCID_ParentTable,_DBCID_Database),LEFT(lcValue,LEN(lcValue)-1),;
       IIF(INLIST(liIDProp,_DBCID_ParentTag,_DBCID_DataSource,;
 	       _DBCID_ConnectString,_DBCID_Password,_DBCID_UserID,;
   	       _DBCID_ParameterList,_DBCID_ConnectName,_DBCID_SQL,;
           _DBCID_Tables,_DBCID_UpdateName,_DBCID_DataType,;
           _DBCID_DisplayClass,_DBCID_DisplayClassLibrary,;
           _DBCID_Format,_DBCID_InputMask,;
           _DBCID_DBCEventFileName),LEFT(lcValue,LEN(lcValue)-1),;
       IIF(INLIST(liIDProp,_DBCID_SubType,_DBCID_DispLogin,;
           _DBCID_Transactions,_DBCID_BatchUpdateCount,_DBCID_SourceType,;
           _DBCID_UpdateType,_DBCID_WhereType),ASC(lcValue),;
       IIF(INLIST(liIDProp,_DBCID_FetchSize,_DBCID_MaxRecords),;
           IIF(lcValue=CHR(255)+CHR(255)+CHR(255)+CHR(255),-1,DFO_C4ToIRev(lcValue)),;
       IIF(INLIST(liIDProp,_DBCID_ConnectTimeout,_DBCID_IdleTimeout,_DBCID_QueryTimeout,;
           _DBCID_WaitTime,_DBCID_PacketSize,_DBCID_UseMemoSize,;
           _DBCID_Version),DFO_C4ToIRev(lcValue),;
       IIF(INLIST(liIDProp,_DBCID_Prim_Cand,_DBCID_Asynchronous,_DBCID_BatchMode,;
           _DBCID_DispWarnings,_DBCID_FetchMemo,_DBCID_ShareConnection,;
           _DBCID_SendUpdates,_DBCID_KeyField,_DBCID_Updatable,;
           _DBCID_CompareMemo,_DBCID_FetchAsNeeded,_DBCID_Offline,;
           _DBCID_Prepared,_DBCID_DBCEvents,_DBCID_AllowSimultaneousFetch,;
           _DBCID_DisconnectRollback),ASC(lcValue)=1,lcValue))))))
ENDPROC

******************************************************************************************
PROCEDURE DBC_ConvertValueToProperty(liIDProp,luValue) && Convert value to DBC value
* liIDProp - Property ID
* luValue  - Property value

*!*	<pdm_sc_yes/>
*!*	<pdm_dd_yes/>

RETURN IIF(INLIST(liIDProp,_DBCID_Comment,_DBCID_Path,;
           _DBCID_RuleExpression,_DBCID_RuleText,_DBCID_InsertTrigger,;
           _DBCID_UpdateTrigger,_DBCID_DeleteTrigger,_DBCID_PrimaryKey,;
           _DBCID_DefaultValue,_DBCID_Caption,_DBCID_ChildTag,;
           _DBCID_ParentTable,_DBCID_Database),luValue+CHR(0),;
       IIF(INLIST(liIDProp,_DBCID_ParentTag,_DBCID_DataSource,;
 	       _DBCID_ConnectString,_DBCID_Password,_DBCID_UserID,;
   	       _DBCID_ParameterList,_DBCID_ConnectName,_DBCID_SQL,;
           _DBCID_Tables,_DBCID_UpdateName,_DBCID_DataType,;
           _DBCID_DisplayClass,_DBCID_DisplayClassLibrary,;
           _DBCID_Format,_DBCID_InputMask,_DBCID_DBCEventFileName),luValue+CHR(0),;
       IIF(INLIST(liIDProp,_DBCID_SubType,_DBCID_DispLogin,;
           _DBCID_Transactions,_DBCID_BatchUpdateCount,_DBCID_SourceType,;
           _DBCID_UpdateType,_DBCID_WhereType),CHR(luValue),;
       IIF(INLIST(liIDProp,_DBCID_FetchSize,_DBCID_MaxRecords),;
           IIF(luValue=-1,CHR(255)+CHR(255)+CHR(255)+CHR(255),DFO_IToC4Rev(luValue)),;
       IIF(INLIST(liIDProp,_DBCID_ConnectTimeout,_DBCID_IdleTimeout,_DBCID_QueryTimeout,;
           _DBCID_WaitTime,_DBCID_PacketSize,_DBCID_UseMemoSize,;
           _DBCID_Version),DFO_IToC4Rev(luValue),;
       IIF(INLIST(liIDProp,_DBCID_Prim_Cand,_DBCID_Asynchronous,_DBCID_BatchMode,;
           _DBCID_DispWarnings,_DBCID_FetchMemo,_DBCID_ShareConnection,_DBCID_DisconnectRollback,;
           _DBCID_SendUpdates,_DBCID_KeyField,_DBCID_Updatable,;
           _DBCID_CompareMemo,_DBCID_FetchAsNeeded,_DBCID_Offline,;
           _DBCID_Prepared,_DBCID_DBCEvents,;
           _DBCID_AllowSimultaneousFetch),IIF(luValue,CHR(1),CHR(0)),luValue))))))
ENDPROC

******************************************************************************************
PROCEDURE DBC_GetDefaultValue(liIDProp) && Get default Value
* liIDProp   - Property ID

*!*	<pdm_sc_yes/>
*!*	<pdm_dd_yes/>

RETURN IIF(INLIST(liIDProp,_DBCID_Comment,_DBCID_Path,;
           _DBCID_RuleExpression,_DBCID_RuleText,_DBCID_InsertTrigger,;
           _DBCID_UpdateTrigger,_DBCID_DeleteTrigger,_DBCID_PrimaryKey,;
           _DBCID_DefaultValue,_DBCID_Caption,_DBCID_ChildTag,;
           _DBCID_ParentTable,_DBCID_Database),"",;
       IIF(INLIST(liIDProp,_DBCID_ParentTag,_DBCID_DataSource,;
 	       _DBCID_ConnectString,_DBCID_Password,_DBCID_UserID,;
   	       _DBCID_ParameterList,_DBCID_ConnectName,_DBCID_SQL,;
           _DBCID_Tables,_DBCID_UpdateName,_DBCID_DataType,;
           _DBCID_DisplayClass,_DBCID_DisplayClassLibrary,;
           _DBCID_Format,_DBCID_InputMask,_DBCID_DBCEventFileName),"",;
       IIF(INLIST(liIDProp,_DBCID_SubType,_DBCID_ConnectTimeout,;
           _DBCID_IdleTimeout,_DBCID_QueryTimeout),0,;
       IIF(INLIST(liIDProp,_DBCID_DispLogin,_DBCID_Transactions,;
           _DBCID_BatchUpdateCount,;
           _DBCID_SourceType,_DBCID_UpdateType,_DBCID_WhereType),1,;
       IIF(liIDProp=_DBCID_MaxRecords,-1,;
       IIF(liIDProp=_DBCID_Version,10,;
       IIF(INLIST(liIDProp,_DBCID_WaitTime,_DBCID_FetchSize),100,;
       IIF(liIDProp=_DBCID_PacketSize,4096,;
       IIF(liIDProp=_DBCID_UseMemoSize,255,;
       IIF(INLIST(liIDProp,_DBCID_DispWarnings,_DBCID_ShareConnection,;
           _DBCID_CompareMemo),.T.,;
       IIF(INLIST(liIDProp,_DBCID_Prim_Cand,_DBCID_Asynchronous,_DBCID_BatchMode,;
           _DBCID_FetchMemo,_DBCID_AllowSimultaneousFetch,_DBCID_DisconnectRollback,;
           _DBCID_SendUpdates,_DBCID_KeyField,_DBCID_Updatable,;
           _DBCID_FetchAsNeeded,_DBCID_Offline,;
           _DBCID_Prepared,_DBCID_DBCEvents),.F.,.NULL.)))))))))))

ENDPROC


******************************************************************************************
PROCEDURE DBC_FormatProperty(liIDProp) && Convert property ID to Name
* liIDProp   - Property ID

*!*	<pdm_sc_yes/>
*!*	<pdm_dd_yes/>

DO CASE
   CASE liIDProp=_DBCID_Path
        RETURN "Path"

   CASE liIDProp=_DBCID_SubType
        RETURN "SubType"

   CASE liIDProp=_DBCID_Comment
        RETURN "Comment"

   CASE liIDProp=_DBCID_RuleExpression
        RETURN "RuleExpression"

   CASE liIDProp=_DBCID_RuleText
        RETURN "RuleText"

   CASE liIDProp=_DBCID_DefaultValue
        RETURN "DefaultValue"

   CASE liIDProp=_DBCID_ParameterList
        RETURN "ParameterList"

   CASE liIDProp=_DBCID_ChildTag
        RETURN "ChildTag"

   CASE liIDProp=_DBCID_InsertTrigger
        RETURN "InsertTrigger"

   CASE liIDProp=_DBCID_UpdateTrigger
        RETURN "UpdateTrigger"

   CASE liIDProp=_DBCID_DeleteTrigger
        RETURN "DeleteTrigger"

   CASE liIDProp=_DBCID_Prim_Cand
        RETURN "Prim_Cand"

   CASE liIDProp=_DBCID_ParentTable
        RETURN "ParentTable"

   CASE liIDProp=_DBCID_ParentTag
        RETURN "ParentTag"

   CASE liIDProp=_DBCID_PrimaryKey
        RETURN "PrimaryKey"

   CASE liIDProp=_DBCID_Version
        RETURN "Version"

   CASE liIDProp=_DBCID_BatchUpdateCount
        RETURN "BatchUpdateCount"

   CASE liIDProp=_DBCID_DataSource
        RETURN "DataSource"

   CASE liIDProp=_DBCID_ConnectName
        RETURN "ConnectName"

   CASE liIDProp=_DBCID_UpdateName
        RETURN "UpdateName"

   CASE liIDProp=_DBCID_FetchMemo
        RETURN "FetchMemo"

   CASE liIDProp=_DBCID_FetchSize
        RETURN "FetchSize"

   CASE liIDProp=_DBCID_KeyField
        RETURN "KeyField"

   CASE liIDProp=_DBCID_MaxRecords
        RETURN "MaxRecords"

   CASE liIDProp=_DBCID_ShareConnection
        RETURN "ShareConnection"

   CASE liIDProp=_DBCID_SourceType
        RETURN "SourceType"

   CASE liIDProp=_DBCID_SQL
        RETURN "SQL"

   CASE liIDProp=_DBCID_Tables
        RETURN "Tables"

   CASE liIDProp=_DBCID_SendUpdates
        RETURN "SendUpdates"

   CASE liIDProp=_DBCID_Updatable
        RETURN "Updatable"

   CASE liIDProp=_DBCID_UpdateType
        RETURN "UpdateType"

   CASE liIDProp=_DBCID_UseMemoSize
        RETURN "UseMemoSize"

   CASE liIDProp=_DBCID_WhereType
        RETURN "WhereType"

   CASE liIDProp=_DBCID_DisplayClass
        RETURN "DisplayClass"

   CASE liIDProp=_DBCID_DisplayClassLibrary
        RETURN "DisplayClassLibrary"

   CASE liIDProp=_DBCID_Format
        RETURN "Format"

   CASE liIDProp=_DBCID_InputMask
        RETURN "InputMask"

   CASE liIDProp=_DBCID_Caption
        RETURN "Caption"

   CASE liIDProp=_DBCID_Asynchronous
        RETURN "Asynchronous"

   CASE liIDProp=_DBCID_BatchMode
        RETURN "BatchMode"

   CASE liIDProp=_DBCID_ConnectString
        RETURN "ConnectString"

   CASE liIDProp=_DBCID_ConnectTimeout
        RETURN "ConnectTimeout"

   CASE liIDProp=_DBCID_DispLogin
        RETURN "DispLogin"

   CASE liIDProp=_DBCID_DispWarnings
        RETURN "DispWarnings"

   CASE liIDProp=_DBCID_IdleTimeout
        RETURN "IdleTimeout"

   CASE liIDProp=_DBCID_QueryTimeout
        RETURN "QueryTimeout"

   CASE liIDProp=_DBCID_Password
        RETURN "Password"

   CASE liIDProp=_DBCID_Transactions
        RETURN "Transactions"

   CASE liIDProp=_DBCID_UserID
        RETURN "UserID"

   CASE liIDProp=_DBCID_WaitTime
        RETURN "WaitTime"

   CASE liIDProp=_DBCID_TimeStamp
        RETURN "TimeStamp"

   CASE liIDProp=_DBCID_DataType
        RETURN "DataType"

   CASE liIDProp=_DBCID_PacketSize
        RETURN "PacketSize"

   CASE liIDProp=_DBCID_Database
        RETURN "Database"

   CASE liIDProp=_DBCID_Prepared
        RETURN "Prepared"

   CASE liIDProp=_DBCID_CompareMemo
        RETURN "CompareMemo"

   CASE liIDProp=_DBCID_FetchAsNeeded
        RETURN "FetchAsNeeded"

   CASE liIDProp=_DBCID_Offline
        RETURN "Offline"

   CASE liIDProp=_DBCID_DBCEvents
        RETURN "DBCEvents"

   CASE liIDProp=_DBCID_DBCEventFileName
        RETURN "DBCEventFileName"

   CASE liIDProp=_DBCID_AllowSimultaneousFetch
        RETURN "AllowSimultaneousFetch"

   CASE liIDProp=_DBCID_DisconnectRollback
        RETURN "DisconnectRollback"

ENDCASE
RETURN "Unknown"

******************************************************************************************
PROCEDURE GetDBCContainer(luFile,lcDBC) && Get DBC Name
* luFile - Handle or file name of DBF file
* @lcDBC - (output) DBC name

IF EMPTY(luFile)
   RETURN _DBC_ErrParamFailed
ENDIF

LOCAL llHandle,lihFile,liFirst,lii
IF TYPE("luFile")="N" && If number (handle)
   llHandle=.T.
   lihFile=luFile
ELSE
   IF TYPE("luFile")="C"  && If string
      lihFile=FOPEN(ALLT(luFile),10) && Open file
      IF lihFile=-1 && If failed  :-(
         RETURN _DBC_ErrNotOpenLowLevelFile && get out
      ENDIF
   ELSE
      RETURN _DBC_ErrParamFailed
   ENDIF
ENDIF

=FSEEK(lihFile,0) && skip to 0. byte
IF ASC(FREAD(lihFile,1))=0x30 && If 0x30
   =FSEEK(lihFile,8) && Skip to 8. byte
   liFirst=ASC(FREAD(lihFile,1)) && Read byte
   liFirst=(liFirst+ASC(FREAD(lihFile,1))*256)-263
   =FSEEK(lihFile,liFirst) && Skip to first byte about SBC name
   lcDBC=FREAD(lihFile,263) && Read 
   lii=AT(CHR(0),lcDBC) && Find first \0
   IF lii>0 && Yes
      lcDBC=LEFT(lcDBC,lii-1) && 
   ENDIF
ENDIF

IF !llHandle && If param wasn't file handle
   =FCLOSE(lihFile) && Close it
ENDIF
RETURN _DBC_ErrOK && 
ENDPROC


******************************************************************************************
PROCEDURE GetTypeDBF(luFile,liType) && Get table type
* luFile - Handle or file name of DBF file
* @liType - (output) Table type

IF EMPTY(luFile)
   RETURN _DBC_ErrParamFailed
ENDIF

LOCAL llHandle,lihFile,liType
liType=0
IF TYPE("luFile")="N" && If number (handle)
   llHandle=.T.
   lihFile=luFile
ELSE
   IF TYPE("luFile")="C"  && If string
      lihFile=FOPEN(ALLT(luFile),10) && Open file
      IF lihFile=-1 && If failed  :-(
         RETURN _DBC_ErrNotOpenLowLevelFile && get out
      ENDIF
   ELSE
      RETURN _DBC_ErrParamFailed
   ENDIF
ENDIF

=FSEEK(lihFile,0) && skip to 0. byte
liType=ASC(FREAD(lihFile,1))

IF !llHandle && If param wasn't file handle
   =FCLOSE(lihFile) && Close it
ENDIF
RETURN liType && Return table type
ENDPROC

******************************************************************************************
PROCEDURE GetReccountDBF(luFile) && Get Record count from DBF header
* luFile - Handle or file name of DBF file

IF EMPTY(luFile)
   RETURN _DBC_ErrParamFailed
ENDIF

LOCAL llHandle,lihFile,liRecc
IF TYPE("luFile")="N" && If number (handle)
   llHandle=.T.
   lihFile=luFile
ELSE
   IF TYPE("luFile")="C"  && If string
      lihFile=FOPEN(ALLT(luFile),10) && Open file
      IF lihFile=-1 && If failed  :-(
         RETURN _DBC_ErrNotOpenLowLevelFile && get out
      ENDIF
   ELSE
      RETURN _DBC_ErrParamFailed
   ENDIF
ENDIF

=FSEEK(lihFile,4) && Skip to 4. byte
liRecc=C4ToI(FREAD(lihFile,4))

IF !llHandle && If param wasn't file handle
   =FCLOSE(lihFile) && Close it
ENDIF
RETURN liRecc && Return record count
ENDPROC



********************************
PROCEDURE DFO_C4ToI(lcInt)  && Convert binnary format to number (32)
RETURN ASC(LEFT(lcInt,1))+;
       ASC(SUBS(lcInt,2,1))*256+;
       ASC(SUBS(lcInt,3,1))*65536+;
       ASC(RIGHT(lcInt,1))*16777216
ENDPROC

********************************
PROCEDURE DFO_C4ToIRev(lcInt)  && Convert binnary format (reverz) to number (32)
RETURN ASC(LEFT(lcInt,1))*16777216+;
       ASC(SUBS(lcInt,2,1))*65536+;
       ASC(SUBS(lcInt,3,1))*256+;
       ASC(RIGHT(lcInt,1))
ENDPROC

                  
                  

******************************************************************************************
PROCEDURE DFO_C2ToI(lcInt)  && Convert binnary format to number (16)
RETURN ASC(LEFT(lcInt,1))+ASC(RIGHT(lcInt,1))*256  
ENDPROC

********************************
PROCEDURE DFO_IToC2(liVal) && Convert number (16) to binnary format
LOCAL lcVal,lii

lii=INT(liVal/256) && Vydl 256
lcVal=CHR(liVal-lii*256) && Peve rozdl na znak a piti ho
liVal=lii && Zapi novou hodnotu pro pevod

lii=INT(liVal/256) && Vydl 256
RETURN lcVal+CHR(liVal-lii*256)
ENDPROC


********************************
PROCEDURE DFO_IToC4(liVal) && Convert number (32) to binnary format
LOCAL lcVal,lii

lii=INT(liVal/256) && Vydl 256
lcVal=CHR(liVal-lii*256) && Peve rozdl na znak a piti ho
liVal=lii && Zapi novou hodnotu pro pevod

lii=INT(liVal/256) && Vydl 256
lcVal=lcVal+CHR(liVal-lii*256)
liVal=lii && Zapi novou hodnotu pro pevod

lii=INT(liVal/256) && Vydl 256
lcVal=lcVal+CHR(liVal-lii*256)
liVal=lii && Zapi novou hodnotu pro pevod

lii=INT(liVal/256) && Vydl 256
RETURN lcVal+CHR(liVal-lii*256) && Peve rozdl na znak a piti ho
ENDPROC

********************************
PROCEDURE DFO_IToC4Rev(liVal) && Convert number (32) to binnary format (reverz)
LOCAL lcVal,lii

lii=INT(liVal/256) && Vydl 256
lcVal=CHR(liVal-lii*256) && Peve rozdl na znak a piti ho
liVal=lii && Zapi novou hodnotu pro pevod

lii=INT(liVal/256) && Vydl 256
lcVal=CHR(liVal-lii*256)+lcVal
liVal=lii && Zapi novou hodnotu pro pevod

lii=INT(liVal/256) && Vydl 256
lcVal=CHR(liVal-lii*256)+lcVal
liVal=lii && Zapi novou hodnotu pro pevod

lii=INT(liVal/256) && Vydl 256
RETURN CHR(liVal-lii*256)+lcVal && Peve rozdl na znak a piti ho
ENDPROC

